/*
//
//       Elvees Performance Platform
//           matrix processing
//
*/

#pragma once

#include "eppm.h"

namespace eppm
{

/*
//  Name:        Transpose
//  Purpose:     Transposes the matrix
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse matrix data pointer
//    pDst       The destination matrix data pointer
//    strideR    Step to the next row
//    strideC    Step to the next element in row
//    srcRows    The number of rows in source matrix
//    srcCols    The number of columns in source matrix
//  Notes:
*/

static inline epp_Status Transpose(const epp_s16* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_s16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_s16(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_s32* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_s32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_s32(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_s64* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_s64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_s64(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cs64* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cs64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cs64(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

//static inline epp_Status Transpose(const epp_f16* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
//                                   epp_f16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
//{
//     return eppm_Transpose_f16(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
//}

static inline epp_Status Transpose(const epp_f32* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_f32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_f32(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_f64* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_f64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_f64(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cs16* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cs16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cs16(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cs32* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cs32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cs32(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cf16* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cf16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cf16(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cf32* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cf32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cf32(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Transpose(const epp_cf64* pSrc, epp_s32 srcStrideR, epp_s32 srcStrideC, epp_size srcRows, epp_size srcCols,
                                   epp_cf64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Transpose_cf64(pSrc, srcStrideR, srcStrideC, srcRows, srcCols, pDst, dstStrideR, dstStrideC);
}


/*
//  Name:        Add
//  Purpose:     Adds two matrixes
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse matrixes data pointer.
//    pDst       The destination matrix data pointer.
//    strideR    Steps to the next row
//    strideC    Steps to the next element in row
//    numRows    The number of rows.
//    numCols    The number of columns.
//  Notes:
*/

static inline epp_Status Add(const epp_f16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_f16(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Add(const epp_f32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_f32(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Add(const epp_f64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_f64(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Add(const epp_cf16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_cf16(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Add(const epp_cf32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_cf32(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Add(const epp_cf64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Add_cf64(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}


/*
//  Name:        Sub
//  Purpose:     Subtracts two matrixes
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse matrixes data pointer.
//    pDst       The destination matrix data pointer.
//    strideR    Steps to the next row
//    strideC    Steps to the next element in row
//    numRows    The number of rows.
//    numCols    The number of columns.
//  Notes:
*/

static inline epp_Status Sub(const epp_f16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_f16(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Sub(const epp_f32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_f32(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Sub(const epp_f64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_f64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_f64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_f64(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Sub(const epp_cf16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_cf16(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Sub(const epp_cf32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_cf32(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}

static inline epp_Status Sub(const epp_cf64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC,
                             const epp_cf64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC,
                             epp_cf64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC,
                             epp_size numRows, epp_size numCols)
{
     return eppm_Sub_cf64(pSrc1, src1StrideR, src1StrideC, pSrc2, src2StrideR, src2StrideC, pDst, dstStrideR, dstStrideC, numRows, numCols);
}


/*
//  Name:        Mul
//  Purpose:     Multiplies two matrixes
//  Context:
//  Returns:     epp_Status
//    epp_StsOK  Ok
//  Parameters:
//    pSrc       The sourse matrixes data pointer.
//    pDst       The destination matrix data pointer.
//    strideR    Steps to the next row
//    strideC    Steps to the next element in row
//    numRows    The number of rows.
//    numCols    The number of columns.
//  Notes:
*/

static inline epp_Status Mul(const epp_s16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_s16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_s16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC, epp_size scale)
{
     return eppm_Mul_s16_NS(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC, scale);
}

static inline epp_Status Mul(const epp_s32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_s32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_s32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC, epp_size scale)
{
     return eppm_Mul_s32_NS(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC, scale);
}

static inline epp_Status Mul(const epp_f16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_f16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_f16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_f16(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Mul(const epp_f32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_f32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_f32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_f32(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Mul(const epp_f64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_f64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_f64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_f64(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Mul(const epp_cs16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_cs16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_cs16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC, epp_size scale)
{
     return eppm_Mul_cs16_NS(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC, scale);
}

static inline epp_Status Mul(const epp_cs32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_cs32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_cs32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC, epp_size scale)
{
     return eppm_Mul_cs32_NS(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC, scale);
}

static inline epp_Status Mul(const epp_cf16* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_cf16* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_cf16* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_cf16(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Mul(const epp_cf32* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_cf32* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_cf32* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_cf32(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

static inline epp_Status Mul(const epp_cf64* pSrc1, epp_s32 src1StrideR, epp_s32 src1StrideC, epp_size src1Rows, epp_size src1Cols,
                             const epp_cf64* pSrc2, epp_s32 src2StrideR, epp_s32 src2StrideC, epp_size src2Cols,
                             epp_cf64* pDst, epp_s32 dstStrideR, epp_s32 dstStrideC)
{
     return eppm_Mul_cf64(pSrc1, src1StrideR, src1StrideC, src1Rows, src1Cols, pSrc2, src2StrideR, src2StrideC, src2Cols, pDst, dstStrideR, dstStrideC);
}

} // namespace eppm
